/*
    fortran.y:

	  Yacc grammar for Fortran program checker.  Uses the yylex()
	  in file FORLEX.C

*/

%{

/*
  fortran.c:

    Copyright (C) 1992 by Robert K. Moniot.
    This program is free software.  Permission is granted to
    modify it and/or redistribute it.  There is no warranty
    for this program.


	    This grammar is ANSI standard-conforming, except for:
		-- complex constant and a few other ambiguities needing
		   significant lookahead cannot be split across lines.

	    Extensions supported:
	        -- Case insensitive.
	 	-- Hollerith constants.
		-- Variable names may be longer than 6 characters.  Also
		   allows underscores and dollar signs in names.
		-- DO ... ENDDO and DO WHILE loop forms allowed.
		-- NAMELIST supported.
		-- TYPE and ACCEPT I/O statements allowed.
		-- Tabs are permitted in input, and (except in character data)
		   expand into blanks up to the next column equal to 1 mod 8.
		-- Type declarations INTEGER*2, REAL*8, etc. are allowed.
		-- IMPLICIT NONE allowed.
	 */

/*  Author: R. Moniot
 *  Date:   August 1988
 *  Last revision: July 1993
 *  Modif   A. Y
 *  Date    June   1995
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "ftnchek.h"
#include "valchek.h"
#include "symtab.h"

	/* The following section is for use with bison-derived
	   parser.  Define alloca to be malloc for those cases
	   not covered by the cases covered there.  The ifdefs
	   are those in the skeleton parser with includes removed */
#ifdef AIXC	/* IBM RS/6000 xlc compiler does it this way */
#pragma alloca
#endif
#ifndef alloca
#ifdef __GNUC__
#else /* Not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__)
#else /* Not sparc */
#ifdef MSDOS
#endif /* MSDOS */
#endif /* Not sparc.  */
#endif /* Not GNU C.  */
#define alloca malloc
#endif /* alloca now defined.  */

void exit();

#ifndef YYDEBUG	/* If not declared otherwise... */
/*int yydebug;	 declare yydebug to satisfy extern in ftnchek.c */
#ifdef DEVELOPMENT
#define YYDEBUG 1		/* For development it is handy */
#else
#define YYDEBUG 0
#endif
#endif

#ifdef DEVELOPMENT
#define DEBUG_PARSER
#endif

int current_datatype,	/* set when parse type_name or type_stmt */
    stmt_sequence_no,   /* set when parsing, reset to 0 at end_stmt */
    control_item_count;	/* count of items in control_info_list */
long current_typesize;	/* for type*len declarations */

extern unsigned prev_stmt_line_num; /* shared with advance */

unsigned true_prev_stmt_line_num;	/* shared with symtab.c */

int current_module_hash = -1,	/* hashtable index of current module name */
    current_module_type = type_UNDECL,
    current_module_size,
    executable_stmt=FALSE,
    prev_stmt_class=0,
		 /* flags for lexer */
    complex_const_allowed=FALSE, /* for help in lookahead for these */
    in_assignment_stmt=FALSE,
    inside_format=FALSE,	/* when inside parens of FORMAT  */
    integer_context=FALSE,	/* says integers-only are to follow */
    prev_goto=FALSE,
    goto_flag=FALSE;	/* if unconditional GOTO was encountered */


/******  AY ajout ************/

PRIVATE char intrn_name[MAX_XTERNAL][8];
PRIVATE int nbintrn;

PRIVATE crp_sblevel *subprog_level, *subprog_level0;

PRIVATE int return_label;
PRIVATE int def_arg_num = 0;
PRIVATE int value_flag;
PRIVATE int len_specification_ok;
PRIVATE int char_type_specif;
PRIVATE int in_read_stmt = 0;
PRIVATE int complex_true = 0;
PRIVATE Token subprogram_token;

int arith_expression_type;
extern char *crp_token_name();

/******  Fin AY ajout ********/

PRIVATE void
END_processing();
		/* Uses of Token fields for nonterminals: */
/*
  1. dim_bound_lists: dimensioning info for arrays:
       token.class = no. of dimensions,
       token.subclass = no. of elements
  2. expressions
       token.value.integer = hash index (of identifier)
       token.class = type_byte = storage_class << 4 + datatype
       token.subclass = flags: CONST_EXPR, LVALUE_EXPR, etc.
  3. common variable lists
       token.subclass = flag: COMMA_FLAG used to handle extra/missing commas
*/

#define SEQ_HEADER   1
#define SEQ_IMPLICIT 2
#define SEQ_SPECIF   3
#define SEQ_STMT_FUN 4
#define SEQ_EXEC     5
#define SEQ_END      6

%}

%token tok_identifier
%token tok_array_identifier
%token tok_label
%token tok_integer_const
%token tok_real_const
%token tok_dp_const
%token tok_complex_const
%token tok_dcomplex_const
%token tok_logical_const
%token tok_string
%token tok_hollerith
%token tok_edit_descriptor
%token tok_letter
%token tok_relop	/* .EQ. .NE. .LT. .LE. .GT. .GE. */
%token tok_AND
%token tok_OR
%token tok_EQV
%token tok_NEQV
%token tok_NOT
%token tok_power	/*   **   */
%token tok_concat	/*   //   */
%token tok_ACCEPT
%token tok_ASSIGN
%token tok_BACKSPACE
%token tok_BLOCK
%token tok_BLOCKDATA
%token tok_BYTE
%token tok_CALL
%token tok_CHARACTER
%token tok_CLOSE
%token tok_COMMON
%token tok_COMPLEX
%token tok_CONTINUE
%token tok_DATA
%token tok_DIMENSION
%token tok_DO
%token tok_DOUBLE
%token tok_DOUBLECOMPLEX
%token tok_DOUBLEPRECISION
%token tok_DOWHILE
%token tok_ELSE
%token tok_ELSEIF
%token tok_END
%token tok_ENDDO
%token tok_ENDFILE
%token tok_ENDIF
%token tok_ENTRY
%token tok_EQUIVALENCE
%token tok_EXTERNAL
%token tok_FILE
%token tok_FORMAT
%token tok_FUNCTION
%token tok_GO
%token tok_GOTO
%token tok_IF
%token tok_IMPLICIT
%token tok_INCLUDE
%token tok_INQUIRE
%token tok_INTEGER
%token tok_INTRINSIC
%token tok_LOGICAL
%token tok_NAMELIST
%token tok_NONE
%token tok_OPEN
%token tok_PARAMETER
%token tok_PAUSE
%token tok_PRECISION
%token tok_PRINT
%token tok_PROGRAM
%token tok_READ
%token tok_REAL
%token tok_RETURN
%token tok_REWIND
%token tok_SAVE
%token tok_STOP
%token tok_SUBROUTINE
%token tok_THEN
%token tok_TO
%token tok_TYPE
%token tok_WHILE
%token tok_WRITE

%token tok_illegal  /* Illegal token unused in grammar: induces syntax error */

%token EOS	127	/* Character for end of statement.  */

%nonassoc tok_relop

%left REDUCE ')'	/* Used at unit_io to force a reduction */


%%
	/*  The following grammar is based on the ANSI manual, diagrams
	 *  of section F.  Numbers in the comments refer to the diagram
	 *  corresponding to the grammar rule.
	 */


/* 1-5 */

prog_body	:	stmt_list
		|	/* empty file */
		;

stmt_list	:	stmt_list_item
		|	stmt_list stmt_list_item
		;


stmt_list_item	:	ordinary_stmt
			{
				/* Create id token for prog if unnamed. */

			  if(curr_stmt_class == tok_END) {
			    END_processing(&($$));
			  }
			  integer_context = FALSE;
			}
 		|	include_stmt
		|	EOS	/* "sticky" EOF for needed delay */
		;

			/* Statements: note that ordering by category
			   of statement is not enforced in the grammar
			   but is deferred to semantic processing.
			 */

ordinary_stmt	:	stmt
		|	end_stmt
		;

stmt		:	tok_label unlabeled_stmt
		|	unlabeled_stmt
		;

unlabeled_stmt	:	subprogram_header
		|	specification_stmt
		|	executable_stmt
		|	restricted_stmt
		|	error EOS
			{
			    complex_const_allowed = FALSE;
			    integer_context = FALSE;
			    in_assignment_stmt = FALSE;
			    yyerrok; /* (error message already given) */
			}
		;

subprogram_header:	prog_stmt
			{
			    current_module_type = type_PROGRAM;
			    current_module_hash = type_PROGRAM;
			}
		|	function_stmt
			{
			    current_module_type = current_datatype;
			    current_module_hash = current_datatype;
			}
		|	subroutine_stmt
			{
			    current_module_type = type_SUBROUTINE;
			    current_module_hash = type_SUBROUTINE;
			}
		|	block_data_stmt
			{
			    current_module_type = type_BLOCK_DATA;
			    current_module_hash = type_BLOCK_DATA;
			}
		;

end_stmt	:	unlabeled_end_stmt
		|	tok_label unlabeled_end_stmt
		;

unlabeled_end_stmt:	tok_END EOS
		;

include_stmt	:	tok_INCLUDE tok_string EOS
 			{
#ifdef ALLOW_INCLUDE
			  if(f77_standard) {
			      nonstandard($1.line_num,$1.col_num);
			  }
 			  open_include_file($2.value.string);
#endif
 			}
 		;

/* 5,6 */
		/* Note that stmt_function_stmt is not distinguished from
		   assignment_stmt, but assign (label to variable) is.
		   Also, format_stmt w/o label is accepted here.
		   ANSI standard for statement sequencing is enforced here. */
specification_stmt:	anywhere_stmt
			{
			     if(stmt_sequence_no < SEQ_IMPLICIT) {
				stmt_sequence_no = SEQ_IMPLICIT;
			     }
			}
		|	parameter_stmt
			{
			     if(stmt_sequence_no < SEQ_IMPLICIT) {
				   stmt_sequence_no = SEQ_IMPLICIT;
			     }
			     else if(stmt_sequence_no > SEQ_SPECIF) {
			     }
			}
		|	implicit_stmt
		|	data_stmt
			{
			     if(stmt_sequence_no < SEQ_STMT_FUN) {
				stmt_sequence_no = SEQ_STMT_FUN;
		 	     }
			}
		|	specif_stmt
		;

anywhere_stmt	:	entry_stmt
			{
			     goto_flag = prev_goto = FALSE;
			}
		|	format_stmt
		;

specif_stmt	:	dimension_stmt
		|	equivalence_stmt
		|	common_stmt
		|	namelist_stmt
		|	type_stmt
		|	external_stmt
		|	intrinsic_stmt
		|	save_stmt
		;


/* 7 */
executable_stmt:		/* Allowed in logical IF */
			transfer_stmt
			{
			    goto_flag=TRUE;
			}
		|	nontransfer_stmt
			{
			    goto_flag=FALSE;
			}
		;

transfer_stmt	:	unconditional_goto
		|	assigned_goto
		|	arithmetic_if_stmt
		|	stop_stmt
		|	return_stmt
		;

nontransfer_stmt:	assignment_stmt
		|	assign_stmt
		|	computed_goto	/* fallthru allowed */
		|	continue_stmt
		|	pause_stmt
		|	read_stmt
		|	accept_stmt
		|	write_stmt
		|	print_stmt
		|       type_output_stmt
		|	rewind_stmt
		|	backspace_stmt
		|	endfile_stmt
		|	open_stmt
		|	close_stmt
		|	inquire_stmt
		|	call_stmt
		;

restricted_stmt:		/* Disallowed in logical IF */
			restricted_nontransfer_stmt
			{
			    goto_flag=FALSE;
			}
		|	else_or_endif_stmt
			{
			    prev_goto = goto_flag =FALSE;
			}
		;

restricted_nontransfer_stmt:
			logical_if_stmt
		|	block_if_stmt
		|	do_stmt
		|	enddo_stmt
		;

else_or_endif_stmt:	else_if_stmt
		|	else_stmt
		|	end_if_stmt
		;

/* 8 */
prog_stmt	:	tok_PROGRAM symbolic_name EOS
			{
			    g_argument_nb = 0;
			    g_line_input = $1.line_num - 1;
                             crp_set_srcname(&($2), type_PROGRAM, 0);
			}
		;

			/* Note that function & subroutine entry not
			 * distinguished in this grammar.
			 */
/* 9 */
entry_stmt	:	tok_ENTRY symbolic_name EOS
			{
			    g_argument_nb = 0;
			    g_line_input = $1.line_num - 1;
			    crp_set_srcname(&($2), current_module_type, current_module_size);
			}
		|	tok_ENTRY symbolic_name
			{
			    g_line_input = $1.line_num - 1;
			}
			'(' dummy_argument_list ')' EOS
			{
                             crp_set_srcname(&($2), current_module_type, current_module_size);
			}
		;

/* 10 */
function_stmt	:	unlabeled_function_stmt
		;

unlabeled_function_stmt
		:	typed_function_handle symbolic_name EOS
			{
			    g_argument_nb = 0;
			    g_line_input = $1.line_num - 1;
			    crp_set_srcname(&($2), current_module_type, current_module_size);
			}
		|	typed_function_handle symbolic_name
			{
			    g_line_input = $1.line_num - 1;
			}
			'(' dummy_argument_list ')' EOS
			{
			     crp_set_srcname(&($2), current_module_type, current_module_size);
			}
		|	plain_function_handle symbolic_name
			EOS
			{
			    g_argument_nb = 0;
			    g_line_input = $1.line_num - 1;
			     crp_set_srcname(&($2), current_module_type, current_module_size);
			}
		|	plain_function_handle symbolic_name
			{
			    g_line_input = $1.line_num - 1;
			}
			'(' dummy_argument_list ')' EOS
			{
			     crp_set_srcname(&($2), current_module_type, current_module_size);
			}
		;

typed_function_handle:	type_name
			function_keyword
			{
			    current_module_type = current_datatype;
			    current_module_size = current_typesize;
			}
		;

plain_function_handle:	function_keyword
			{
			    current_module_type = 0;
			    current_module_size = size_DEFAULT;
			}
		;

function_keyword:	tok_FUNCTION
		;

type_name	:	arith_type_name
		|	plain_char_type_name
		|	char_type_name
		;


/* 11 not present: see 9 */

/* 12 */
subroutine_stmt	:	unlabeled_subroutine_stmt
		;

unlabeled_subroutine_stmt
		:	subroutine_handle symbolic_name EOS
			{
			    g_argument_nb = 0;
			    g_line_input = $1.line_num - 1;
			    crp_set_srcname(&($2), type_SUBROUTINE, size_SUBROUTINE);
			}
		|	subroutine_handle symbolic_name
			{
			    g_line_input = $1.line_num - 1;
			    strcpy(g_subprog_name,crp_token_name($2));
			}
			'(' dummy_argument_list ')' EOS
			{
			      crp_set_srcname(&($2), type_SUBROUTINE, size_SUBROUTINE);
			}
		;

subroutine_handle:	tok_SUBROUTINE
			{current_module_type=type_SUBROUTINE;}
		;

dummy_argument_list:	/* empty */
			{
			    g_argument_nb = 0;
			    def_arg_num = -1;
			    $$.next_token = (Token*)NULL;
			}
		|	non_empty_arg_list
                        {
                            g_argument_nb = def_arg_num+1;
                        }

		;

non_empty_arg_list:	dummy_argument
			{
			    def_arg_num = 0;
			    crp_get_dummy_argument(def_arg_num,&($1), return_label);
			}
		|	non_empty_arg_list ',' dummy_argument
			{
			    def_arg_num++;
			    crp_get_dummy_argument(def_arg_num,&($3), return_label);
			}
		;

dummy_argument	:	symbolic_name
			{ return_label = FALSE; }
		|	'*'
			{ return_label = TRUE;}
		;

/* 13 not present: see 9 */

/* 14 */
block_data_stmt	:	block_data_handle EOS
			{
			    g_line_input = $1.line_num - 1;
			    crp_set_srcname(NULL, type_BLOCK_DATA, -1000);
			}
		|	block_data_handle symbolic_name EOS
			{
			    g_line_input = $1.line_num - 1;
			    crp_set_srcname(&($2), type_BLOCK_DATA, size_SUBROUTINE);
			}
		;

block_data_handle:	tok_BLOCK tok_DATA
		|	tok_BLOCKDATA
		;
/* 15 */
dimension_stmt	:	tok_DIMENSION array_declarator_list EOS
		;

array_declarator_list:	array_declarator
		|	array_declarator_list ',' array_declarator
		;

/* 16 */
array_declarator:	symbolic_name '(' dim_bound_list ')'
			{ crp_set_array_var_bd(&($1), g_length_array); }
		;

dim_bound_list	:	dim_bound_item      /* token class = no. of dimensions,
					       subclass = no. of elements */
			{
			        g_length_array = $1.value.integer;
			}
		|	dim_bound_list ',' dim_bound_item
			{
			        g_length_array *= $3.value.integer;
			}
		;

dim_bound_item	:	dim_bound_expr
		|	dim_bound_expr ':' dim_bound_expr
			{
			  $$.value.integer=$3.value.integer-$1.value.integer+1;
			}
		|	'*'
		|	dim_bound_expr ':' '*'
		;

/* 17 */
equivalence_stmt:	tok_EQUIVALENCE {equivalence_flag = TRUE;}
			equivalence_list EOS {equivalence_flag = FALSE;}
		;

equivalence_list:	'(' equivalence_list_item ')'
		|	equivalence_list ',' '(' equivalence_list_item ')'
		;

equivalence_list_item:	equiv_entity ',' equiv_entity
		|	equivalence_list_item ',' equiv_entity
		;

/* 17 */
equiv_entity	:	symbolic_name
		|	array_equiv_name
		|	substring_equiv_name
		;

array_equiv_name:	symbolic_name '(' subscript_list ')'
				/* should check */
		;

substring_equiv_name:	symbolic_name substring_interval
		|	array_equiv_name substring_interval
		;

/* 19 */
common_stmt	:	tok_COMMON common_variable_list EOS
		|	tok_COMMON common_block_list EOS
		|	tok_COMMON common_variable_list common_block_list EOS
		;

	/*  The following defns allow trailing commas and missing commas in
	    order to tolerate the optional comma before /blockname/.  The
	    token subclass holds comma status to allow errors to be caught. */
common_block_list:	labeled_common_block
		|	common_block_list labeled_common_block
			{
			     $$.line_num = $2.line_num;
			     $$.col_num = $2.col_num;
			}
		;

labeled_common_block:	common_block_name {g_entity_nb=0;} common_variable_list
			{
			     $$.line_num = $3.line_num;
			     $$.col_num = $3.col_num;
			     crp_mem_common_bd(&($1));
			}
		;

common_block_name:	'/' symbolic_name '/'
			{
			     $$ = $2;
			}

		|	'/'  '/'		/* block with no name */
		|	tok_concat		/* "//" becomes this */
		;

common_variable_list:	common_list_item
			{
			    $$.subclass = $1.subclass;
			}
		|	common_variable_list common_list_item
			{
			    $$.line_num = $2.line_num;
			    $$.col_num = $2.col_num;
			}
		;

common_list_item:	common_entity
		|	common_entity ','
		;

common_entity	:	symbolic_name
			{
                            crp_set_common_type_bd(&($1), &g_entity_nb);
			}
		|	array_declarator
			{
                            crp_set_common_type_bd(&($1), &g_entity_nb);
			}
		;


/* NAMELIST : Not Standard
   Syntax is:
	NAMELIST /group/ var [,var...] [[,] /group/ var [,var...]...]
*/

namelist_stmt	:	tok_NAMELIST namelist_list EOS
		;

namelist_list	:	namelist_decl
		|	namelist_list namelist_decl
		;

namelist_decl	:	namelist_name namelist_var_list
		;

namelist_name	:	'/' symbolic_name '/'
			{
			    $$ = $2;
			}
		;

namelist_var_list:	namelist_item
		|	namelist_var_list namelist_item
			{
			    $$.line_num = $2.line_num;
			    $$.col_num = $2.col_num;
			}
		;

namelist_item	:	symbolic_name
		|	symbolic_name ','
		;

/* 20 */
type_stmt	:	arith_type_name arith_type_decl_list EOS
		|	plain_char_type_name char_type_decl_list EOS
		|	char_type_name char_type_decl_list EOS
		|	char_type_name ',' char_type_decl_list EOS
		;

arith_type_name	:	sizeable_type_name
			{
			  current_typesize = type_size[current_datatype];
			}
				/* Allow *len to modify some arith types */
		|	sizeable_type_name '*' nonzero_unsigned_int_const
			{
			    current_typesize = $3.value.integer;
#if 0 /* defunct feature */
			    if(local_wordsize > 0) {
			      /*  recognize REAL*2w as DOUBLE PRECISION */
			      if(current_datatype == type_REAL
				 && $3.value.integer == type_size[type_DP])
				current_datatype = type_DP;
			      /*  recognize COMPLEX*4w as DOUBLE COMPLEX */
			      if(current_datatype == type_COMPLEX
				 && $3.value.integer==type_size[type_DCOMPLEX])
				current_datatype = type_DCOMPLEX;
			    }
#endif
			}
				/* Other type disallow *len modifier */
		|	unsizeable_type_name
		;

sizeable_type_name:	tok_INTEGER
			{
			     current_datatype = type_INTEGER;
			     integer_context = TRUE;
			}
		|	tok_REAL
			{
			     current_datatype = type_REAL;
			     integer_context = TRUE;
			}
		|	tok_COMPLEX
			{
			     current_datatype = type_COMPLEX;
			     integer_context = TRUE;
			}
		|	tok_LOGICAL
			{
			     current_datatype = type_LOGICAL;
			     integer_context = TRUE;
			}
		;

unsizeable_type_name:	tok_DOUBLE tok_PRECISION
			{
			     current_datatype = type_DP;
			     current_typesize = size_DEFAULT;
			}
		|	tok_DOUBLEPRECISION
			{
			     current_datatype = type_DP;
			     current_typesize = size_DEFAULT;
			}
		|	tok_DOUBLE tok_COMPLEX
			{
			     current_datatype = type_DCOMPLEX;
			     current_typesize = size_DEFAULT;
			}
		|	tok_DOUBLECOMPLEX
			{
			     current_datatype = type_DCOMPLEX;
			     current_typesize = size_DEFAULT;
			}
		|	tok_BYTE /* treate BYTE as a form of integer for now */
			{
			     current_datatype = type_INTEGER;
			     current_typesize = 1;
			}
		;

plain_char_type_name:	tok_CHARACTER
			{
			     current_datatype = type_STRING;
			     current_typesize = 1;
			     len_specification_ok = 0;
			     integer_context = TRUE;
			}
		;

char_type_name	:	plain_char_type_name '*' len_specification
			{
			     current_typesize = $3.value.integer;
			}
		;

arith_type_decl_list:	arith_type_decl_item
		|	arith_type_decl_list ',' arith_type_decl_item
		;

arith_type_decl_item:	symbolic_name
			{
			    crp_set_var_type_bd(&($1), current_datatype,
				current_typesize, $1.line_num);
			}
		|	array_declarator
			{
			    crp_set_var_type_bd(&($1), current_datatype,
				current_typesize, $1.line_num);
			}
		;

char_type_decl_list:	char_type_decl_item
		|	char_type_decl_list ',' char_type_decl_item
		;

char_type_decl_item:	symbolic_name
			{
                            crp_set_var_type_bd(&($1), current_datatype,  
                                current_typesize, $1.line_num);
			}
		|	symbolic_name '*' len_specification
			{
                            crp_set_var_type_bd(&($1), current_datatype, 
                                current_typesize, $1.line_num);
			}
		|	array_declarator
			{
                            crp_set_var_type_bd(&($1), current_datatype,
                                current_typesize, $1.line_num);
			}
		|	array_declarator '*' len_specification
			{
                            crp_set_var_type_bd(&($1), current_datatype,
                                current_typesize, $1.line_num);
			}
   		;

/* 21 */
				/* implicit_flag helps is_keyword's work */
implicit_handle	:	tok_IMPLICIT {implicit_flag=TRUE;}
		;

implicit_stmt	:	implicit_handle implicit_decl_list EOS
				/* IMPLICIT NONE statement */
		|	implicit_handle tok_NONE EOS
		;

implicit_decl_list:	implicit_decl_item
		|	implicit_decl_list ',' {initial_flag = TRUE;}
				       implicit_decl_item
		;

		/* implicit_letter_flag tells lexer to treat letters as letters,
			   not as identifiers */
implicit_decl_item:	type_name '('  {implicit_letter_flag = TRUE;}
				letter_list ')'  {implicit_letter_flag = FALSE;}
		;

letter_list	:	letter_list_item
		|	letter_list ',' letter_list_item
		;

letter_list_item:	tok_letter
			{
			   int c1 = (int)$1.subclass;
			   set_implicit_type(current_datatype,current_typesize,
                                        c1,c1);
			}
		|	tok_letter '-' tok_letter
			{
			   int c1 = (int)$1.subclass;
			   int c2 = (int)$3.subclass;
			   set_implicit_type(current_datatype,current_typesize,
                                        c1,c2);
			}
		;


/* 22 */
len_specification:	'(' '*' ')'
			    {$$.value.integer = size_ADJUSTABLE;len_specification_ok = 1;}
		|	nonzero_unsigned_int_const
			    {$$.value.integer = $1.value.integer;len_specification_ok = 0;}
		|	'(' int_constant_expr ')'
			    {
                                if ( value_flag )
				{
				    $$.value.integer = $2.value.integer;
				}
				else
				{
				    $$.value.integer = crp_x_value_lu(&($2));
				}
				len_specification_ok = 0;
			    }
		;

/* 23 */
parameter_stmt	:	tok_PARAMETER '(' parameter_defn_list ')' EOS
   		;

parameter_defn_list:	parameter_defn_item
		|	parameter_defn_list ',' parameter_defn_item
		;

parameter_defn_item:	symbolic_name {complex_const_allowed = TRUE;} '='
				parameter_expr
			{
			    complex_const_allowed = FALSE;
			    crp_mem_value_lu(&($1), crp_x_value(&($4)));
			}
		;

/* 24 */
external_stmt	:	tok_EXTERNAL external_name_list EOS
		;

external_name_list:	symbolic_name
			{
                             if ( g_nbextrn < MAX_XTERNAL )
                             {
                                 strcpy(g_extrn_name[g_nbextrn], crp_token_name($1));
                                 g_nbextrn++;
                             } 
			}
		|	external_name_list ',' symbolic_name
                        { 
                             if ( g_nbextrn < MAX_XTERNAL ) 
                             {
                                 strcpy(g_extrn_name[g_nbextrn], crp_token_name($3)); 
                                 g_nbextrn++; 
                             }  
                        }
		;

/* 25 */
intrinsic_stmt	:	tok_INTRINSIC intrinsic_name_list EOS
		;

intrinsic_name_list:	symbolic_name
			{
                            if ( nbintrn < MAX_XTERNAL )
                            {
                                strcpy(intrn_name[nbintrn], crp_token_name($1));
                                nbintrn++;
                            }
			}
		|	intrinsic_name_list ',' symbolic_name
			{
                            if ( nbintrn < MAX_XTERNAL )
                            {
                                strcpy(intrn_name[nbintrn], crp_token_name($3));
                                nbintrn++;
                            }
                        }
		;

/* 26 */
save_stmt	:	tok_SAVE EOS
			{
			  global_save = TRUE;
			}
		|	tok_SAVE save_list EOS
		;

save_list	:	save_item
		|	save_list ',' save_item
		;

save_item	:	symbolic_name
		|	'/' symbolic_name '/'
		;

/* 27 */
data_stmt	:	tok_DATA data_defn_list EOS
   		;

data_defn_list	:	data_defn_item
		|	data_defn_list data_defn_item
		|	data_defn_list ',' data_defn_item
		;

data_defn_item	:	data_defn_assignee_list '/'
				{complex_const_allowed=TRUE;}
				data_value_list
				{complex_const_allowed=FALSE;} '/'
		;

data_defn_assignee_list
		:	data_defn_assignee
		|	data_defn_assignee_list ',' data_defn_assignee
		;

data_defn_assignee:	lvalue
		|	data_implied_do_list
		;

data_value_list:	data_value
		|	data_value_list ',' data_value
		;

data_value	:	data_constant_value
		|	data_repeat_factor '*' data_constant_value
		;

data_repeat_factor:	nonzero_unsigned_int_const
		|	symbolic_name
		;

data_constant_value:	data_constant
		|	symbolic_name
		;


data_dlist	:	data_dlist_item
		|	data_dlist ',' data_dlist_item
		;

data_dlist_item	:	array_element_lvalue
		|	data_implied_do_list
		;

data_implied_do_list:  '(' data_dlist ',' symbolic_name
				'=' data_do_loop_bounds ')'
		;

data_do_loop_bounds:	int_constant_expr ',' int_constant_expr
		| int_constant_expr ',' int_constant_expr ',' int_constant_expr
		;


/* 29 */
assignment_stmt	:	lvalue '=' {complex_const_allowed = TRUE;
                                    in_assignment_stmt = TRUE;} expr
			{
			  complex_const_allowed = FALSE;
                          in_assignment_stmt = FALSE;
			  crp_output_set(&($1), $1.line_num);
			}
				 EOS
		;

lvalue		:	variable_name
		|	array_element_lvalue
		|	substring_lvalue
		|	stmt_function_handle
		;


/* array-element_lvalue is at 88 */

assign_stmt	:    	tok_ASSIGN pre_label label tok_TO variable_name EOS
		;


/* 31 */
unconditional_goto:	goto pre_label label EOS
		;

/* 32 */
computed_goto	:	goto '(' goto_list ')' integer_expr EOS
		|	goto '(' goto_list ')' ',' integer_expr EOS
		;

/* 33 */
assigned_goto	:	goto symbolic_name EOS
		|	goto symbolic_name '(' goto_list ')' EOS
		|	goto symbolic_name ',' '(' goto_list ')' EOS
		;

goto		:	tok_GOTO
			{
                            integer_context=TRUE;
			}
		|	tok_GO tok_TO
			{
                            integer_context=TRUE;
			}
		;

goto_list	:	pre_label label
		|	goto_list ',' pre_label label
		;

/* 34 */
arithmetic_if_stmt:	if_handle pre_label label ',' pre_label label
				 ',' pre_label label EOS
		;

/* 35 */
logical_if_stmt	:	if_handle executable_stmt
		;

/* 36 */
block_if_stmt	:	if_handle tok_THEN EOS
		;

if_handle	:	tok_IF '('  {complex_const_allowed = TRUE;} expr ')'
			{complex_const_allowed = FALSE;initial_flag = TRUE;}
		;

/* 37 */
else_if_stmt	:	tok_ELSE block_if_stmt
		|	tok_ELSEIF '(' {complex_const_allowed = TRUE;} expr ')'
			{complex_const_allowed = FALSE;initial_flag = TRUE;}
			tok_THEN EOS
		;

/* 38 */
else_stmt	:	tok_ELSE EOS
		;

/* 39 */
end_if_stmt	:	tok_ENDIF EOS
		|	tok_END tok_IF EOS
		;

/* 40 */
			/* Allow VAX/VMS extensions:
			   DO [label [,]] var = expr , expr [,expr]
			   DO [label [,]] WHILE ( expr )
			      ...
			   ENDDO
			*/

do_stmt		:	do_handle variable_name
				'=' do_loop_bounds EOS
		|	do_handle tok_WHILE '('
			        {complex_const_allowed=TRUE;}
				expr ')' EOS
				{complex_const_allowed=FALSE;}
		|	tok_DOWHILE '('
				{complex_const_allowed=TRUE;}
				expr ')' EOS
				{complex_const_allowed=FALSE;}
		;

do_handle	:	tok_DO pre_label label
		|	tok_DO pre_label label ','
		|	tok_DO pre_label
			{integer_context=FALSE;}
		;

do_loop_bounds	:	int_real_dp_expr ',' int_real_dp_expr
		|   int_real_dp_expr ',' int_real_dp_expr ',' int_real_dp_expr
		;

enddo_stmt	:	tok_END tok_DO EOS
		|	tok_ENDDO EOS
		;

/* 41 */
continue_stmt	:	tok_CONTINUE EOS
		;

/* 42 */
stop_stmt	:	tok_STOP stop_info EOS
		;

/* 43 */
pause_stmt	:	tok_PAUSE stop_info EOS
		;

stop_info	:	/* empty */
		|	tok_integer_const
		|	symbolic_name
		|	tok_string
		;

/* 44 */
write_stmt	:	write_handle
				{complex_const_allowed = FALSE;} EOS
		|	write_handle io_list
				{complex_const_allowed = FALSE;} EOS
		;

write_handle	:	tok_WRITE {control_item_count = 0;}
				'(' control_info_list ')'
				{complex_const_allowed = TRUE;}
		;

/* 45 */
		/* Note that parenthesized format_id's will end up in
		   control_info_list. Disambiguation left to semantic phase.
		   This is why we need the optional comma */
read_stmt	:	read_handle '(' control_info_list ')' EOS
			{in_read_stmt=0;}
		|	read_handle '(' control_info_list ')' io_list EOS
			{in_read_stmt=0;}
		|	read_handle '(' control_info_list ')' ',' io_list EOS
			{in_read_stmt=0;}
		|	read_handle format_id EOS
			{in_read_stmt=0;}
		|	read_handle format_id ',' io_list EOS
			{in_read_stmt=0;}
		;
read_handle	:	tok_READ {control_item_count = 0;in_read_stmt=1;}
		;

accept_stmt	:	tok_ACCEPT format_id EOS
		|	tok_ACCEPT format_id ',' {in_read_stmt=1;} io_list {in_read_stmt=0;} EOS
		;

/* 46 */
print_stmt	:	tok_PRINT format_id EOS
   		|	tok_PRINT format_id ',' io_list EOS
		;

type_output_stmt:	tok_TYPE format_id EOS
   		|	tok_TYPE format_id ','
				{complex_const_allowed = TRUE;} io_list
				{complex_const_allowed = FALSE;} EOS
		;

/* 47 */
control_info_list:	control_info_item
		|	control_info_list ',' control_info_item
		;

	/* Note that unit id is not distinguished from format id
	   by the grammar. Use sequence no. to tell which is which.
	 */
control_info_item:	symbolic_name '=' unit_id
		|	unit_id
		;

			/* OPEN stmt needs its own control list defn to
			   allow for VMS READONLY and similar keywords.
			   Special prodn for unit_id as optional 1st item
			   needed to avoid reduce/reduce conflict with
			   later-occurring symbolic_name items.   */
open_info_list	:	unit_id
		|	symbolic_name '=' unit_id
		|	open_info_list ',' open_info_item
		;

open_info_item	:	symbolic_name '=' unit_id
		|	symbolic_name	/* NOSPANBLOCKS, READONLY or SHARED */
		;

/* 48 */
io_list		:	io_item
		|	io_list ',' io_item
		;

io_item		:	expr
		        {
			    if ( in_read_stmt )
			    {
				crp_output_set(&($1), $1.line_num);
			    }
			}
		|	io_implied_do_list
		;

/* 49 */
io_implied_do_list:	'(' io_list ',' variable_name '=' do_loop_bounds ')'
		;

/* 50 */
open_stmt	:	tok_OPEN {control_item_count = 0;}
				 '(' open_info_list ')' EOS
		;

/* 51 */
close_stmt	:	tok_CLOSE {control_item_count = 0;}
				'(' control_info_list ')' EOS
		;

/* 52 */
inquire_stmt	:	tok_INQUIRE {control_item_count = 0;}
				'(' control_info_list ')' EOS
		;

/* 53 */
backspace_stmt	:	backspace_handle unit_id EOS
		|	backspace_handle '(' control_info_list ')' EOS
		;
backspace_handle:	tok_BACKSPACE {control_item_count = 0;}
		;

/* 54 */
endfile_stmt	:	endfile_handle unit_id EOS
		|	endfile_handle '(' control_info_list ')' EOS
		;
endfile_handle	:	tok_ENDFILE {control_item_count = 0;}
		|	tok_END tok_FILE {control_item_count = 0;}
		;

/* 55 */
rewind_stmt	:	rewind_handle unit_id EOS
		|	rewind_handle '(' control_info_list ')' EOS
		;
rewind_handle	:	tok_REWIND {control_item_count = 0;}
		;


/* 56 */
		/* "expr" causes shift/reduce conflict on ')' between
		   red'n  unit_id: expr_  and shift  primary: ( expr_ ).
		   Use "associativity" rule to force reduction */
unit_id		:	expr		%prec REDUCE
		|	'*'
		;

/* 57 */
format_id	:	char_expr
		|	'*'
		;

/* 58,59 */
format_stmt	:	tok_FORMAT {inside_format=TRUE;} '(' format_spec ')' EOS
			{
			    inside_format=FALSE;
			}
		;

/* 60-69 */
format_spec	:		/* EMPTY */
		|	nonempty_format_spec
		;


nonempty_format_spec:	fmt_spec_item
		|	nonempty_format_spec fmt_spec_item
		;

fmt_spec_item	:	repeatable_fmt_item
		|	unrepeatable_fmt_item
		|	fmt_item_separator
		;

repeatable_fmt_item:	'(' nonempty_format_spec ')'
		|	tok_edit_descriptor
		;

unrepeatable_fmt_item:	tok_string
		|	tok_hollerith
		|	repeat_spec
		|	variable_fmt_item
		;

fmt_item_separator:	','
		|	'/'
		|	tok_concat	/* since lexer spots "//" */
		|	':'
		|	'.'		/* Occurs when variable w.d is used */
		|	nonstandard_fmt_item
		;

nonstandard_fmt_item: '$'	/* VMS uses this */
		;

repeat_spec	:	tok_integer_const
		|	'-' tok_integer_const	/* for kP descriptor */
		|	'+' tok_integer_const	/* for +kP descriptor */
		;

		/* VMS-style variable format size or repeat spec*/
variable_fmt_item:	'<'  integer_expr '>'
		;

/* 70 handle only: complete defn handled as assignment stmt */

stmt_function_handle:	scalar_name '(' stmt_function_dummy_list ')'
			{
				crp_val_stmt_function(&($1));
			}
		;

stmt_function_dummy_list: /* empty list */
		| nonempty_stmt_fun_dummy_list
		;

nonempty_stmt_fun_dummy_list:	  stmt_function_dummy_arg
		|	  nonempty_stmt_fun_dummy_list ','
					stmt_function_dummy_arg
		;

stmt_function_dummy_arg:  variable_name	/* for now: later, handle correctly */
		;

/* 71 */
call_stmt	:	call_handle {complex_const_allowed = FALSE;} EOS
			{
			    crp_update_argument(subprog_level);
			    free(subprog_level);
			}
		|	call_handle '(' ')' {complex_const_allowed = FALSE;}
						EOS
			{
			    crp_update_argument(subprog_level);
			    free(subprog_level);
			}
		|	call_handle '(' expr_list ')'
				{complex_const_allowed = FALSE;} EOS
			{
			    crp_update_argument(subprog_level);
			    free(subprog_level);
			}
		;

call_handle	:	tok_CALL symbolic_name
			{
			     subprog_level = (crp_sblevel *)malloc(	
					sizeof(crp_sblevel));
			     subprog_level->call_arg_num = 0;
			     subprog_level->prev = NULL;
			     $$ = $2;
   			     crp_set_subprog_carac(subprog_level, &($2));
			     complex_const_allowed = TRUE;
			}
		;
expr_list	:	expr
			{
			    if ( subprog_level->normal_subprogram )
			    {
			        if ( value_flag )
			        {
				  crp_mem_value(subprog_level, $1.line_num);
			        }
				crp_actual_arg_set_val( subprog_level,&($1),
					$1.line_num, value_flag);
			    }
			    subprog_level->call_arg_num++;
			}
		|	'*' pre_label label
			{
			    subprog_level->call_arg_num++;
			}
		|	expr_list ',' expr
			{
			    if ( subprog_level->normal_subprogram )
			    {
			        if ( value_flag )
                                {
				  crp_mem_value(subprog_level, $3.line_num);
			        }
			        crp_actual_arg_set_val( subprog_level,&($3),
					$3.line_num, value_flag);
			    }
			    subprog_level->call_arg_num++;
			}
		|	expr_list ',' '*' pre_label label
			{
			    subprog_level->call_arg_num++;
			}
		;

/* 72 */
return_stmt	:	tok_RETURN EOS
		|	tok_RETURN integer_expr EOS
		;

/* 73 */
function_reference:	fun_or_substr_handle '(' fun_arg_list ')'
			{
   			    crp_set_subprog_carac1(subprog_level);
			    if ( subprog_level->ptype != NULL )
			    {
                                if ( *subprog_level->ptype < type_SUBROUTINE )
			        {
			            *subprog_level->ptype += type_SUBROUTINE;
				}
			    }
			    crp_update_argument(subprog_level);
			    crp_func_ref_expr_val(subprog_level->subprogram_name,
					&($3), &($$));
			    subprog_level0 = subprog_level->prev;
			    free(subprog_level);
			    subprog_level= subprog_level0;
			    complex_const_allowed=FALSE;
			    in_assignment_stmt = TRUE;
			}
		;

fun_or_substr_handle:	scalar_name
			{
			  subprog_level0 = (crp_sblevel *)malloc(sizeof(crp_sblevel));
                          subprog_level0->ptype = NULL;
			  subprog_level0->prev = subprog_level;
			  subprog_level = subprog_level0;
			  subprog_level->call_arg_num = 0;
                          crp_set_subprog_carac0(subprog_level, &($1));
			  if ( complex_const_allowed )
			  {
			    complex_true = 1;
			  }
			  complex_const_allowed=TRUE;
			  in_assignment_stmt = FALSE;
			}
		;
fun_arg_list	:	/* empty */
		|	nonempty_fun_arg_list
		;

nonempty_fun_arg_list:	expr
			{
			    if ( subprog_level->normal_subprogram )
			    {
			        if ( value_flag )
			 	{
				   crp_mem_value(subprog_level, $1.line_num);
			    	}
			        crp_actual_arg_set_val( subprog_level,&($1),
					$1.line_num, value_flag);
			    }
			    subprog_level->call_arg_num++;
			}
		|	nonempty_fun_arg_list ',' expr
			{
			    if (subprog_level->normal_subprogram )
			    {
			        if ( value_flag )
				{
				  crp_mem_value(subprog_level, $3.line_num);
			    	}
			        crp_actual_arg_set_val( subprog_level,&($3),
					$3.line_num, value_flag);
			    }
			    subprog_level->call_arg_num++;
			}

/* 74 not present: type checking not done at this level */

/* 75 was constant_expr, but only used by PARAMETER */
parameter_expr	:	/* arith, char, or logical */ expr
		;

/* 76 following the text of the standard, not the diagrams */
expr		:	log_expr
		;

log_expr	:	log_disjunct

		|	expr tok_EQV log_disjunct
			{
                            value_flag = 1;
			}
		|	expr tok_NEQV log_disjunct
			{
                            value_flag = 1;
			}
		;

log_disjunct	:	log_term

		|	log_disjunct tok_OR log_term
			{
                            value_flag = 1;
			}
		;

log_term	:	log_factor

		|	log_term tok_AND log_factor
			{
                            value_flag = 1;
			}
		;

log_factor	:	log_primary

		|	tok_NOT log_primary
			{
                            value_flag = 1;
			}
		;

log_primary	:	arith_expr

		|	log_primary tok_relop log_primary
			{
                            value_flag = 1;
			}
		;


arith_expr	:	term

		|	'-' term
			{
                            value_flag = 1;
			    crp_unexpr_type_val(&($2),&($$));
			    if ( arith_expression_type == type_INTEGER )
			    {
			        $$.value.integer = -($2.value.integer);
			    }
			}
		|	'+' term
			{
/*
			    crp_unexpr_type_val(&($2),&($$));
*/
			    $$ = $2;
                            value_flag = 1;
			}
		|	arith_expr '+' term
			{
			    crp_binexpr_type_val(&($1), &($2), &($3), &($$));
                            value_flag = 1;
			}
		|	arith_expr '-' term
			{
			    crp_binexpr_type_val(&($1), &($2), &($3), &($$));
                            value_flag = 1;
			}
		;

term		:	factor

		|	term '/' factor
			{
                            value_flag = 1;
			    crp_binexpr_type_val(&($1), &($2), &($3), &($$));
			}
		|	term '*' factor
			{
			    crp_binexpr_type_val(&($1), &($2), &($3), &($$));
                            value_flag = 1;
			}
		;

factor		:	char_expr
		|	char_expr tok_power factor
			{
                            value_flag = 1;
			}
		;

char_expr	:	primary

		|	char_expr tok_concat primary
			{
                            crp_concat_string(&($1), &($3), &($$));
                            value_flag = 3;
			}
		;

primary		:	variable_name
                        { value_flag = 0; }

		|	array_element_name
                        { value_flag = 0; }

		|	function_reference
                        { value_flag = 0; }

		|	substring_name
                        {
			    value_flag = 0;
			}
		|	literal_const
			{
                            value_flag = 1;
			}
		|	'(' expr ')'
			{
			    $$ = $2;
				/* (identifier) becomes a non-identifier */
                            value_flag = 1;
			}
		;

				/* Literal constants are numbers, strings
				   holleriths, and logical constants */
literal_const	:	numeric_const
			{
			    /* (class is set in numeric_const productions) */
			    $$.size = size_DEFAULT;
			}
		|	tok_string
			{
                            $$.class = type_byte(class_VAR,type_STRING);
			}
		|	tok_hollerith
			{
                            $$.class = type_byte(class_VAR,type_HOLLERITH);
			}
		|	tok_logical_const
			{
			    $$.size = size_DEFAULT;
                            $$.class = type_byte(class_VAR,type_LOGICAL);
			}
		;

numeric_const	:	tok_integer_const
			{
                            $$.class = type_byte(class_VAR,type_INTEGER);
                        }
		|	tok_real_const
			{
                            $$.class = type_byte(class_VAR,type_REAL);
                        }
		|	tok_dp_const
			{
                            $$.class = type_byte(class_VAR,type_DP);
                        }
		|	tok_complex_const
			{
                            $$.class = type_byte(class_VAR,type_COMPLEX);
                        }
		|	tok_dcomplex_const
			{
                            $$.class = type_byte(class_VAR,type_DCOMPLEX);
                        }
		;

/* 77 */
integer_expr	:	/* integer */ arith_expr
		;

/* 78 */
int_real_dp_expr:	/* integer, real, or double */ arith_expr
		;

/* 79 absent */

/* 80 */
int_constant_expr:	/* integer const */ arith_expr
		;

/* 81 */
dim_bound_expr	:       /* integer */  arith_expr
			{
			    $$.value.integer = crp_x_value(&($1));
/*
			    if ( value_flag )
			    {
				$$.value.integer = $1.value.integer;
			    }
			    else
			    {
				$$.value.integer = crp_x_value_lu(&($1));
			    }
*/
			}
		;

/* 82-85 absent: no type checking here */
/* 86-87 absent: see 76 */

/* 88 */
array_element_lvalue:	array_name '(' subscript_list ')'
		;

array_element_name:	array_name '(' subscript_list ')'
		;

subscript_list	:	subscript
		|	subscript_list ',' subscript
		     ;

subscript	:	expr
		;

/* 89 */
substring_name	:	fun_or_substr_handle  substring_interval
			{
			    subprog_level0 = subprog_level->prev;
			    free(subprog_level);
			    subprog_level= subprog_level0;
                            if (! complex_true )
			    {
			        complex_const_allowed=FALSE;
			    }
			    complex_true = 0;
			    $$.size=substring_size_val(&($1),&($2));
			}

		|	array_element_name substring_interval
			{
                            $$.size=substring_size_val(&($1),&($2));
			}
		;

substring_lvalue:	scalar_name substring_interval
			{
                            $$.size=substring_size_val(&($1),&($2));
                        }
		|	array_element_lvalue substring_interval
			{
                            $$.size=substring_size_val(&($1),&($2));
                        }
		;
			/* substring interval: limits go into class,subclass */
substring_interval:	'(' ':' ')'
			{
                            $$.class=1;
                            $$.subclass=0; /* 0 means LEN */
			}
		  |	'(' substr_index_expr ':' ')'
			{
                            $$.class=$2.value.integer;
                            $$.subclass=0; /* 0 means LEN */
                        }
		  |	'(' ':' substr_index_expr ')'
                        {
                            $$.class=1;
                            $$.subclass=$3.value.integer;
                        }
		  |	'(' substr_index_expr ':' substr_index_expr ')'
                        {
                              $$.class=$2.value.integer;
                              $$.subclass=$4.value.integer;
                        }
		  ;

substr_index_expr:	arith_expr
		;

/* 90-98 absent: name categories not distinguished */

/* 99 */
variable_name	:	scalar_name
		|	array_name
		;

scalar_name	:	tok_identifier
		;

array_name	:	tok_array_identifier
		;


/* symbolic_name refers to a name without making it into an id expr */
symbolic_name	:	tok_identifier
		|	tok_array_identifier
		;

/* 100 */
data_constant	:	numeric_const
		|	'-' numeric_const
		|	'+' numeric_const
		|	tok_logical_const
   		|	tok_string
		|	tok_hollerith
		;

/* 101-102 absent */

/* 103 */
nonzero_unsigned_int_const:
			tok_integer_const
		;

/* 104-109 absent: lexer handles these */
	/* pre_label prepares for an expected label by setting flag
	   so that lexer won't look for E-format number.  All grammar
	   rules that have "label" precede it with "pre_label" */
pre_label	:	/* NOTHING */
			{
			    integer_context=TRUE;
			}
		;

/* 110 */
label		:	tok_integer_const
			{
				integer_context=FALSE;
			}
		;

/* 111-116 absent: lexer handles these */

%%
void
init_parser()			/* Initialize various flags & counters */
{
	initial_flag = TRUE;	/* set flag for keyword test */
	implicit_flag=FALSE;	/* clear flags for IMPLICIT stmt */
	implicit_letter_flag = FALSE;
	implicit_type_given = FALSE;
	implicit_none = FALSE;
	global_save = FALSE;
	complex_const_allowed = FALSE;
	prev_token_class = EOS;
	stmt_sequence_no = 0;
	true_prev_stmt_line_num = 0;
}

/* After having parsed prog_stmt, function_stmt, subroutine_stmt,
   block_data_stmt, the stmt_sequence_no is set to the value SEQ_HEADER.
*/

	/* After having parsed end_stmt, common block lists and
	   subprogram argument lists are copied over into global symbol
	   table, the local symbol table is printed out and then cleared,
	   and stmt_sequence_no is set to zero for start of next module.
	*/

PRIVATE void
END_processing(t)
	Token *t;
{
  crp_end_processing_val();
  if(current_module_hash != -1) {
	{
/*
            process_lists(current_module_hash);
*/
	}
        init_symtab();
  }
  /*  Initialisation pour les verifications de valeurs */
  stmt_sequence_no = 0;
  current_module_hash = -1;
  implicit_type_given = FALSE;
  implicit_none = FALSE;
  in_assignment_stmt = FALSE;
  true_prev_stmt_line_num = 0;
  integer_context = FALSE;
  global_save = FALSE;

  nbintrn = 0;
  g_nbextrn = 0;
}
