      SUBROUTINE GUNM2R( SIDE, TRANS, M, N, K, A, LDA, TAU, C, LDC,
     $                   WORK, INFO )
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF UTILIFOR  DATE 17/02/2003   AUTEUR NICOLAS O.NICOLAS 
C ======================================================================
C COPYRIGHT (C) LAPACK
C ======================================================================
C  -- LAPACK ROUTINE (VERSION 2.0) --
C     UNIV. OF TENNESSEE, UNIV. OF CALIFORNIA BERKELEY, NAG LTD.,
C     COURANT INSTITUTE, ARGONNE NATIONAL LAB, AND RICE UNIVERSITY
C     SEPTEMBER 30, 1994
C
C  PURPOSE
C  =======
C
C  GUNM2R OVERWRITES THE GENERAL COMPLEX M-BY-N MATRIX C WITH
C
C        Q * C  IF SIDE = 'L' AND TRANS = 'N', OR
C
C        Q'* C  IF SIDE = 'L' AND TRANS = 'C', OR
C
C        C * Q  IF SIDE = 'R' AND TRANS = 'N', OR
C
C        C * Q' IF SIDE = 'R' AND TRANS = 'C',
C
C  WHERE Q IS A COMPLEX UNITARY MATRIX DEFINED AS THE PRODUCT OF K
C  ELEMENTARY REFLECTORS
C
C        Q = H(1) H(2) . . . H(K)
C
C  AS RETURNED BY ZGEQRF. Q IS OF ORDER M IF SIDE = 'L' AND OF ORDER N
C  IF SIDE = 'R'.
C
C  ARGUMENTS
C  =========
C
C  SIDE    (INPUT) CHARACTER*1
C          = 'L': APPLY Q OR Q' FROM THE LEFT
C          = 'R': APPLY Q OR Q' FROM THE RIGHT
C
C  TRANS   (INPUT) CHARACTER*1
C          = 'N': APPLY Q  (NO TRANSPOSE)
C          = 'C': APPLY Q' (CONJUGATE TRANSPOSE)
C
C  M       (INPUT) INTEGER
C          THE NUMBER OF ROWS OF THE MATRIX C. M >= 0.
C
C  N       (INPUT) INTEGER
C          THE NUMBER OF COLUMNS OF THE MATRIX C. N >= 0.
C
C  K       (INPUT) INTEGER
C          THE NUMBER OF ELEMENTARY REFLECTORS WHOSE PRODUCT DEFINES
C          THE MATRIX Q.
C          IF SIDE = 'L', M >= K >= 0;
C          IF SIDE = 'R', N >= K >= 0.
C
C  A       (INPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,K)
C          THE I-TH COLUMN MUST CONTAIN THE VECTOR WHICH DEFINES THE
C          ELEMENTARY REFLECTOR H(I), FOR I = 1,2,...,K, AS RETURNED BY
C          ZGEQRF IN THE FIRST K COLUMNS OF ITS ARRAY ARGUMENT A.
C          A IS MODIFIED BY THE ROUTINE BUT RESTORED ON EXIT.
C
C  LDA     (INPUT) INTEGER
C          THE LEADING DIMENSION OF THE ARRAY A.
C          IF SIDE = 'L', LDA >= MAX(1,M);
C          IF SIDE = 'R', LDA >= MAX(1,N).
C
C  TAU     (INPUT) COMPLEX*16 ARRAY, DIMENSION (K)
C          TAU(I) MUST CONTAIN THE SCALAR FACTOR OF THE ELEMENTARY
C          REFLECTOR H(I), AS RETURNED BY ZGEQRF.
C
C  C       (INPUT/OUTPUT) COMPLEX*16 ARRAY, DIMENSION (LDC,N)
C          ON ENTRY, THE M-BY-N MATRIX C.
C          ON EXIT, C IS OVERWRITTEN BY Q*C OR Q'*C OR C*Q' OR C*Q.
C
C  LDC     (INPUT) INTEGER
C          THE LEADING DIMENSION OF THE ARRAY C. LDC >= MAX(1,M).
C
C  WORK    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION
C                                   (N) IF SIDE = 'L',
C                                   (M) IF SIDE = 'R'
C
C  INFO    (OUTPUT) INTEGER
C          = 0: SUCCESSFUL EXIT
C          < 0: IF INFO = -I, THE I-TH ARGUMENT HAD AN ILLEGAL VALUE
C
C  =====================================================================
C-----------------------------------------------------------------------
C ASTER INFORMATION
C 14/01/2000 TOILETTAGE DU FORTRAN SUIVANT LES REGLES ASTER,
C            REMPLACEMENT DE 1 RETURN PAR GOTO 1000,
C            IMPLICIT NONE.
C-----------------------------------------------------------------------
C CORPS DU PROGRAMME
      IMPLICIT NONE
C
C
C     .. SCALAR ARGUMENTS ..
      CHARACTER*1        SIDE, TRANS
      INTEGER            INFO, K, LDA, LDC, M, N
C     ..
C     .. ARRAY ARGUMENTS ..
      COMPLEX*16         A( LDA, * ), C( LDC, * ), TAU( * ), WORK( * )
C     ..
C     .. PARAMETERS ..
      COMPLEX*16         ONE
      PARAMETER          ( ONE = ( 1.0D+0, 0.0D+0 ) )
C     ..
C     .. LOCAL SCALARS ..
      LOGICAL            LEFT, NOTRAN
      INTEGER            I, I1, I2, I3, IC, JC, MI, NI, NQ
      COMPLEX*16         AII, TAUI
C     ..
C     .. EXTERNAL FUNCTIONS ..
      LOGICAL            LLSAME
C     ..
C     .. EXECUTABLE STATEMENTS ..
C
C     TEST THE INPUT ARGUMENTS
C
      INFO = 0
      LEFT = LLSAME( SIDE, 'L' )
      NOTRAN = LLSAME( TRANS, 'N' )
C
C     NQ IS THE ORDER OF Q
C
      IF( LEFT ) THEN
         NQ = M
      ELSE
         NQ = N
      END IF
      IF( .NOT.LEFT .AND. .NOT.LLSAME( SIDE, 'R' ) ) THEN
         INFO = -1
      ELSE IF( .NOT.NOTRAN .AND. .NOT.LLSAME( TRANS, 'C' ) ) THEN
         INFO = -2
      ELSE IF( M.LT.0 ) THEN
         INFO = -3
      ELSE IF( N.LT.0 ) THEN
         INFO = -4
      ELSE IF( K.LT.0 .OR. K.GT.NQ ) THEN
         INFO = -5
      ELSE IF( LDA.LT.MAX( 1, NQ ) ) THEN
         INFO = -7
      ELSE IF( LDC.LT.MAX( 1, M ) ) THEN
         INFO = -10
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'GUNM2R', -INFO )
         GOTO 1000
      END IF
C
C     QUICK RETURN IF POSSIBLE
C
      IF( M.EQ.0 .OR. N.EQ.0 .OR. K.EQ.0 )
     $   GOTO 1000
C
      IF( ( LEFT .AND. .NOT.NOTRAN .OR. .NOT.LEFT .AND. NOTRAN ) ) THEN
         I1 = 1
         I2 = K
         I3 = 1
      ELSE
         I1 = K
         I2 = 1
         I3 = -1
      END IF
C
      IF( LEFT ) THEN
         NI = N
         JC = 1
      ELSE
         MI = M
         IC = 1
      END IF
C
      DO 10 I = I1, I2, I3
         IF( LEFT ) THEN
C
C           H(I) OR H(I)' IS APPLIED TO C(I:M,1:N)
C
            MI = M - I + 1
            IC = I
         ELSE
C
C           H(I) OR H(I)' IS APPLIED TO C(1:M,I:N)
C
            NI = N - I + 1
            JC = I
         END IF
C
C        APPLY H(I) OR H(I)'
C
         IF( NOTRAN ) THEN
            TAUI = TAU( I )
         ELSE
            TAUI = DCONJG( TAU( I ) )
         END IF
         AII = A( I, I )
         A( I, I ) = ONE
         CALL GLARF( SIDE, MI, NI, A( I, I ), 1, TAUI, C( IC, JC ), LDC,
     $               WORK )
         A( I, I ) = AII
   10 CONTINUE
 1000 CONTINUE
C
C     END OF GUNM2R
C
      END
