#!?PYTHON_EXE?
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

# $Id:  $
# $Name:  $

import sys
# ----- check for Python version
if sys.hexversion < 0x020300F0:
   print _('This script requires Python 2.3 or higher, sorry !')
   sys.exit(4)

import os
import os.path
import re
import glob
import shutil
from optparse import OptionParser

# ----- differ messages translation
def _(mesg): return mesg

#-------------------------------------------------------------------------------
# 0. initialisation, configuration of the command-line parser
#-------------------------------------------------------------------------------
# 0.1. ----- Gibi/Castem configuration
# This variable MUST be set in environment for Gibi2003
CASTEM='?HOME_GIBI?'

# original script
gibi=os.path.join(CASTEM, 'gibi2003')

# 0.2. ----- command line parser
usage="""Script to run gibi for Code_Aster.

1) run Gibi in a temporary subdirectory (used by EXEC_LOGICIEL Code_Aster command)
   gibi [options] fileIN fileOUT

2) run Gibi in current directory in interactive mode
   gibi.x [options] [fileIN [fileOUT]]
 or
   gibi --interact [options] [fileIN [fileOUT]]

3) directly open a Gibi/Castem result file (previously saved by SAUV FORMAT)
   gibi_rest [options] fileRES
 or
   gibi --rest [options] [fileIN [fileOUT]]

arguments:
   fileIN   : input file of Gibi
   fileOUT  : destination of fort.8 (default result file of Gibi)
   fileRES  : result file (saved by SAUV FORMAT command) to read

"""

parser = OptionParser(usage=usage)
parser.add_option("-m", "--mem", dest="mem_gibi", action='store',
   help=_("allows to choose the amount of memory used (passed to the original Gibi script), " \
   "possible values for SIZE are S, N (default), M, L, XL, XXL"),
   metavar="SIZE")
parser.add_option("--help-gibi", dest="help_gibi", action='store_true',
   default=False,
   help=_("print help informations given by the original Gibi"))
parser.add_option("-e", "--exec", dest="mode", action='store_const',
   const=1,
   help=_("choose EXEC_LOGICIEL  mode (1). This is the default mode."))
parser.add_option("-i", "--interact", dest="mode", action='store_const',
   const=2,
   help=_("choose INTERACTIVE mode (2)"))
parser.add_option("-r", "--rest", dest="mode", action='store_const',
   const=3,
   help=_("choose RESTITUTION mode (3)"))

opts, args = parser.parse_args()

args_gibi=[]
mem_gibi=opts.mem_gibi
if opts.help_gibi==True:
   args_gibi.append('-h')

# 0.3. ----- running mode
mode=opts.mode
if mode==None:
   mode=1
   base=os.path.basename(sys.argv[0])
   if base=='gibi.x':
      mode=2
   elif base=='gibi_rest':
      mode=3

# 0.4. ----- absolute filenames
args=[os.path.abspath(p) for p in args]

#-------------------------------------------------------------------------------
# 1. Execution
#-------------------------------------------------------------------------------
# 1.1. ----- temporary folder
if mode==1:
   dtmp='tmp_gibi'
   if not os.path.isdir(dtmp):
      os.mkdir(dtmp)
   os.chdir(dtmp)

# 1.2. ----- arguments
ftmp='ftmp'+str(os.getpid())+'.datg'
use_ftmp=False
dest=os.pardir
if len(args)==0:
   use_ftmp=True
   f=open(ftmp,'w')
   f.close()
   src=os.path.abspath(ftmp)
if len(args)>=1:
   src=args[0]
if len(args)==2:
   dest=args[1]

if not os.path.exists(src):
   print _('<F>_ERREUR File not found : %s') % src
   sys.exit(4)

# 1.3. ----- call Gibi
os.environ['CASTEM']=CASTEM
if mode==1:
   cmd=[gibi,] + args_gibi + ['<',src]
elif mode==2:
   cmd=[gibi,] + args_gibi + [src]
elif mode==3:
   use_ftmp=True
   f=open(ftmp,'w')
   f.write('OPTI REST FORMAT\n')
   f.write("'"+src+"';\n")
   f.write('REST FORMAT;\n')
   f.close()
   cmd=[gibi,] + args_gibi + [ftmp]

iret=os.system(' '.join(cmd))


#-------------------------------------------------------------------------------
# 2. Results
#-------------------------------------------------------------------------------
# 2.1. ----- copying result file
if os.path.exists('fort.8'):
   try:
      shutil.copy('fort.8', dest)
   except:
      print _('<F>_ERREUR Failed during copying %s to %s') % ('fort.8', dest)
      iret=6

# 2.2. ----- clean
if use_ftmp:
   os.remove(ftmp)

if mode==1:
   os.chdir(os.pardir)
   shutil.rmtree(dtmp)
else:
   for f in ['UTILPROC', 'UTILNOTI']+glob.glob('fort.*'):
      if os.stat(f).st_size==0 or not f in ['UTILPROC', 'UTILNOTI']:
         try:
            os.remove(f)
         except OSError:
            pass

sys.exit(iret)
