#! /bin/bash 
############################################################################
# Contitution d'une base d'intercompilation CRP pour une version donnee
#
# usage NEW_BASE_REFE version rep_resultat fic_msg
#
############################################################################

if [ $# != 3 ] ; then
   echo "usage : NEW_BASE_REFE version rep_resultat fic_msg"
   exit 4
fi

##################
# INITIALISATION #
##################

iret=0

. ?DABIN?/init_crp_crs.sh

VERSION=$1
DIRRES=$2
if [ ! -d $DIRRES ] ; then
  mkdir -p $DIRRES
fi  
FICMSG=$3

NUM_VER=$(echo $VERSION | cut -c 4,4)
CH_VERSION=$VERSION

mkdir -p $TMPDIR
DTMP=$TMPDIR/NBR
DSUR=$DTMP/SUR
DCRP=$DTMP/CRP

DCRP_VERS=$BASE_ASTER/$CH_VERSION/crp
BIB_FORT=$BASE_ASTER/$CH_VERSION/bibfor
BIB_FERM=$BASE_ASTER/$CH_VERSION/fermetur
BIB_FANT=$DCRP_VERS/fantome
DCRP_PARA=$DCRP_VERS/param

mkdir $DTMP
mkdir $DSUR
mkdir $DCRP

# Liste des erreurs rendant la base inconsistante
cat >$DCRP/errfat <<eof
CRP_999  : erreur syntaxique dans l'outil crp
CRP_998  : interface d'appel inconnu
CRP_996  : sp en double
CRP_101  : nb d'arguments incorrect
CRP_103  : subroutine appelee comme une function
CRP_104  : function appelee comme une subroutine
CRP_105  : function appelee avec un mauvais type
eof

###'

echo "=== new_base_ref $1 $2 $3 $date" >$FICMSG

#####################################
# Source de referece avec surcharge #
#####################################
if [ ! -d $BIB_FERM ] ; then
  echo "" >>$FICMSG
  echo " Rpertoire $BIB_FERM inexistant" >>$FICMSG
  exit 4
else
  echo "" >>$FICMSG
  echo "=== Copie de $BIB_FERM" >>$FICMSG
fi
ls $BIB_FERM | while read -r unit
do
   cp $BIB_FERM/$unit $DSUR/.
done

if [ ! -d $BIB_FORT ] ; then
  echo "" >>$FICMSG
  echo " Rpertoire $BIB_FORT inexistant" >>$FICMSG
  exit 4
else
  echo "" >>$FICMSG
  echo "=== Surcharge avec $BIB_FORT" >>$FICMSG
fi
ls $BIB_FORT | while read -r repe
do
  ls $BIB_FORT/$repe | while read -r unit
  do
    cp $BIB_FORT/$repe/$unit $DSUR/.
  done
done

if [ ! -d $BIB_FANT ] ; then
  echo "" >>$FICMSG
  echo " Rpertoire $BIB_FANT inexistant" >>$FICMSG
  exit 4
else
  echo "" >>$FICMSG
  echo "=== Copie de $BIB_FANT" >>$FICMSG
fi
cp $BIB_FANT/*/*.f $DSUR/.
echo " " >$DCRP/fort.f
ls $DSUR | while read -r unit
do
   cat $DSUR/$unit >>$DCRP/fort.f
done


##########################
# Creation base avec crp #
##########################
echo "" >>$FICMSG
echo "=== Constitution base avec crp" >>$FICMSG
cp $DCRP_PARA/* $DCRP/.
cd $DCRP
$DABIN/crp.sh -repinter $DCRP -new -m $DCRP/fort.f >$DCRP/msg_crp

iret=$?

# Recherche des erreurs detectees par crp
grep '^CRP_' $DCRP/msg_crp | awk '{print $1,$3}' | sort -u >$DCRP/err_crp_ftn

# Fichier des erreurs rendant la base inconsitante
awk '{print $1}' $DCRP/errfat | sort -u >$DCRP/errfat_s

# Erreurs rendant la base inconsistante dans le source
join -1 1 -2 1 $DCRP/err_crp_ftn $DCRP/errfat_s >$DCRP/errfat_crp

if [ -s $DCRP/errfat_crp ] ; then
   echo "" >>$FICMSG
   echo "=== Les erreurs suivantes rendent la base inconsistante :" >>$FICMSG
   cat $DCRP/errfat_crp >>$FICMSG
   iret=4
else
   iret=0
fi

echo "" >>$FICMSG
echo "=== Messages crp :" >>$FICMSG
cat $DCRP/msg_crp >>$FICMSG
 
if [ $iret == 0 ] ; then
   echo "" >>$FICMSG
   echo "=== Copie des fichiers base dans $DIRRES" >>$FICMSG
   cp $DCRP/routine.tmp  $DIRRES/routine
   cp $DCRP/common.tmp   $DIRRES/common
   cp $DCRP/argument.tmp $DIRRES/argument
else
   cp $DCRP/routine.tmp  $DIRRES/routine.tmp
   cp $DCRP/common.tmp   $DIRRES/common.tmp
   cp $DCRP/argument.tmp $DIRRES/argument.tmp
fi
rm -rf $TMPDIR
exit $iret
